// platform: linux
'use strict';

const moduleName = 'native/linux';

const path     = require('path');
const electron = require('electron');
const app      = electron.app;
const helper   = require(path.join(app.getAppPath(), 'helper'));
const winston  = require('winston');

const shortcut  = `${ app.getName() }.desktop`;
const autostart = path.join(app.getPath('appData'), 'autostart');

const shortcutPath = path.join(autostart, shortcut);
const disabledPath = path.join(app.getPath('userData'), shortcut);

var isLaunchedAtLogin = new Promise(function(resolve, reject) {
  function yes() { resolve(true); }
  function no() { resolve(false); }

  helper.stat(shortcutPath).then(yes).catch(no);
});

function toggleLaunchAtLogin() {
  return new Promise(function(resolve, reject) {
    function disable() {
      helper.move(shortcutPath, disabledPath).then(resolve).catch(reject);
    }
    function enable() {
      helper.move(disabledPath, shortcutPath).then(resolve).catch(reject);
    }

    helper.stat(shortcutPath).then(disable).catch(enable);
  });
}

module.exports = function() {
  this.isLaunchedAtLogin   = isLaunchedAtLogin;
  this.toggleLaunchAtLogin = toggleLaunchAtLogin;

  return this;
}
