'use strict';

const moduleName = 'native/win32';

const path     = require('path');
const electron = require('electron');
const app      = electron.app;
const helper   = require(path.join(app.getAppPath(), 'helper'));
const winston  = require('winston');

const shortcut  = `${ app.getName() }.lnk`;
const autostartPath = path.join(app.getPath('appData'), 'Microsoft', 'Windows',
                                'Start Menu', 'Programs', 'Startup');

const enabledPath = path.join(autostartPath, shortcut);
const disabledPath = path.resolve(app.getPath('exe'), '..', shortcut);

function disable() {
  return helper.unlink(enabledPath);
}
function enable() {
  return helper.copy(disabledPath, enabledPath);
}

var isLaunchedAtLogin = new Promise(function(resolve, reject) {
  function yes() { resolve(true); }
  function no() { resolve(false); }

  helper.stat(enabledPath).then(yes).catch(no);
});

function toggleLaunchAtLogin() {
  return new Promise(function(resolve, reject) {
    helper.stat(enabledPath).then(disable, enable).then(resolve, reject);
  });
}

module.exports = function() {
  this.isLaunchedAtLogin   = isLaunchedAtLogin;
  this.toggleLaunchAtLogin = toggleLaunchAtLogin;
  this.downloadURL = 'http://packages2.protonet.info/protosync/current/protosync-setup.exe';

  return this;
}
