/* jshint esversion: 6, node: true, unused: true */
'use strict';

const app     = require('app');
const winston = require('winston');

var queue = Promise.resolve();
var ready = {
  when: function(executor) {
    function task() { return new Promise(executor); }
    queue = queue.then(task);

    return ready;
  },

  then:     undefined,
  catch:    undefined,
  finalize: undefined
};

function errorHandler(reason) {
  winston.error('[ready] Error during initialization!', { reason: reason });
}

var promise = new Promise(function(resolve, reject) {
  winston.debug('[ready] Initializing...');

  ready.then = function(onFullfillment, onRejection) { // jshint ignore:line
    promise = promise.then.apply(promise, arguments);
    return ready;
  };

  ready.catch = function(onRejection) { // jshint ignore:line
    var args = Array.prototype.slice.call(arguments);
    promise.catch.apply(promise, args);

    return ready;
  };

  ready.finalize = function() {
    winston.debug('[ready] Finalizing...');

    app.on('ready', function() {
      winston.debug('[ready] Application initialized.');
      queue.then(resolve, reject);
    });

    return ready;
  };
}).catch(errorHandler);

module.exports = ready;
