var Tray = require('tray');
var ContextMenu = require('./context_menu').ContextMenu;
var ContextMenuItem = require('./context_menu').ContextMenuItem;
var fs = require('fs');
var path = require('path');
var dialog = require("dialog");

var ProtoTray = function(options) {
  this.options = options;
  this.protoPath = options.protoPath;
  this.syncFolderPath = options.syncFolderPath;
  this.buildTray();
  this.buildMenu();
  this._observe();
};

ProtoTray.prototype = {
  buildTray: function() {
    if (process.platform === 'darwin') {
      this.appIcon = new Tray(this.protoPath + "/gfx/favicon_new.png");
      this.appIcon.setPressedImage(this.protoPath + "/gfx/favicon_new_white.png");
    } else {
      this.appIcon = new Tray(this.protoPath + "/gfx/favicon_new_white.png");
      this.appIcon.setPressedImage(this.protoPath + "/gfx/favicon_new.png");
    }
    this.appIcon.setToolTip('Protonet Sync App');
  },

  buildMenu: function() {
    this.contextMenu = new ContextMenu(this.appIcon);

    this.openSyncFolderItem = new ContextMenuItem({
      label: "Open Sync Folder",
      click: function() {
        this.options.openSyncFolder();
      }.bind(this)
    }).addTo(this.contextMenu);

    this.logoutItem = new ContextMenuItem({
      label: "Logout",
      click: function() {
        this.options.login();
      }.bind(this)
    }).addTo(this.contextMenu);

    this.separatorItem = new ContextMenuItem({
      type: "separator"
    }).addTo(this.contextMenu);

    this.quitItem = new ContextMenuItem({
      label: "Quit",
      click: function() {
        this.options.shutdown();
      }.bind(this)
    }).addTo(this.contextMenu);

    this.contextMenu.refresh();
  },

  loggedOutState: function() {
    this.loggedIn = false;
    this.logoutItem.disable();
    this.openSyncFolderItem.disable();
    this.contextMenu.refresh();
  },

  loggedInState: function() {
    this.loggedIn = true;
    this.logoutItem.enable();
    this.openSyncFolderItem.enable();
    this.contextMenu.refresh();
  },

  _observe: function() {
    // OSX only
    this.appIcon.on("drop-files", function(event, files) {
      // setTimeout needed to ensure correct cursor styling
      setTimeout(function() {
        if (!this.loggedIn) {
          dialog.showErrorBox("Login required", "Please login first.");
          return false;
        }

        for (var i=0; i<files.length; i++) {
          this.move(files[i]);
        }
      }.bind(this), 0);
    }.bind(this));
  },

  move: function(filePath) {
    try {
      var statFrom = fs.statSync(filePath);
    } catch(e) {
      return;
    }

    var baseName = path.basename(filePath);
    var targetPath = path.join(this.syncFolderPath, baseName);

    try {
      var statTo = fs.statSync(targetPath)
    } catch(e) {}

    if (statTo && statTo.isDirectory() === statFrom.isDirectory()) {
      // Show dialog
      var type = statTo.isDirectory() ? "folder" : "file"
      var returnValue = dialog.showMessageBox({
        buttons:  ["Replace " + type, "Cancel"],
        type:     "question",
        title:    "Already exists",
        message:  "Your Protosync folder already contains a " + type + " named '" + baseName + "'. Do you want to replace it?"
      });
      if (returnValue === 0) {
        this._move(filePath, targetPath);
      }
    } else {
      this._move(filePath, targetPath);
    }
  },
  
  _move: function(filePath, targetPath) {
    console.log("move", filePath, "to", targetPath);
    fs.rename(filePath, targetPath, function(err) {
      if (err) {
        console.log("error moving:", err);
        // TODO: Error message
      }
    });
  }
};

exports.ProtoTray = ProtoTray;