/* jshint esversion: 6, jquery: true, node: true, unused: true */
/* globals app: true, btoa: true  */
'use strict';

app.controller("LoginCtrl", function($scope, $http, $q, $timeout) {
  const electron  = require('electron');
  const ipc       = electron.ipcRenderer;
  const remote    = require('remote');
  const protosync = remote.app;
  const path      = require('path');
  const home      = protosync.getPath('home');
  const shell     = require("shell");
  const dialog    = require('electron').remote.dialog;


  $scope.page = "boxname";
  $scope.host = "";
  $scope.boxName = "";
  $scope.email = "";
  $scope.password = "";
  $scope.syncPath  = path.join(home, protosync.getName());
  $scope.user_username = "";
  $scope.user_email = "";

  $scope.checkHost = function() {
    $scope.host = parseHost($scope.boxName);
    if ($scope.host.length < 6) {
      return;
    }

    $scope.checking = true;
    $scope.hostExists = false;

    checkHost($scope.host).then(function() {
      $scope.hostExists = true;
    }, function() {
      $scope.hostExists = false;
    }).finally(function() {
      $scope.checking = false;
    });
  };

  const Menu = remote.Menu;

  var ctxMenuTemplate = [{
    label: 'Paste',
    accelerator: 'CmdOrCtrl+V',
    role: 'paste'
  }];
  var ctxMenu = Menu.buildFromTemplate(ctxMenuTemplate);

  $scope.showContextMenu = function() {
    var currentWindow = remote.getCurrentWindow();
    ctxMenu.popup(currentWindow);
  };

  function checkHost(host) {
        return $http.head(`https://${ host }/robots.txt`);
  }

  function parseHost(boxName) {
    return $.trim(boxName)
      .replace(/https?\:\/\//, "")
      .replace(/\/.+/, "")
      .replace(/\.protonet\.de$/, ".protonet.info")
      .toLowerCase();
  }

  function getMe() {
    var deferred = $q.defer();
    $http.get("https://" + $scope.host + "/api/v1/me", {
      headers: {
        "X-Requested-With": "XMLHttpRequest",
        "Authorization":    "Basic " + btoa($scope.email + ":" + $scope.password)
      }
    }).then(function(response) {
      deferred.resolve(response.data.me);
    }).catch(function(response) {
      deferred.reject(response);
    });
    return deferred.promise;
  }

  $scope.next = function() {
    if (!$scope.hostExists) {
      return;
    }
    $scope.page = "login";
  };

  $scope.login = function() {
    $scope.error = null;
    $scope.loading = true;
    getMe().then(function(user) {
      if (user.role === "guest") {
        // TODO: Nicer error message
        $scope.error = "You have the guest role. Guests cannot use sync functionality.";
        $scope.loading = false;
        return;
      }

      $scope.user_username = user.username;
      $scope.user_email = user.email;
      $scope.page = "done";
    }, function() {
      $scope.loading = false;
      $scope.error = "Incorrect email or password.";
    });
  };

  $scope.setSyncFolderLocation = function() {
    console.log("using default syncfolder " + $scope.syncPath);
    $scope.error = null;
    $scope.loading = true;

    // just use default folder location
    var data = {
      boxname:  $scope.host,
      protosyncthingpath: $scope.syncPath,
      username: $scope.user_username,
      email:    $scope.user_email,
      password: $scope.password
    };

    ipc.send("credentials", JSON.stringify(data));

    $scope.page = "folderdone";
  }

  $scope.custom_folder = function() {
    $scope.error = null;
    $scope.loading = true;

    if (process.platform === 'darwin') { 
        dialog.showMessageBox({type: "info", title: "Custom folder location not supported in OSX (yet)", message:"Syncing into a custom folder is not yet supported in OSX, ProtoSync will use the default location.", buttons: ["OK"]});
    } else {
      $scope.syncPath = dialog.showOpenDialog({ title: "Set ProtoSync location", properties: ['openDirectory', 'createDirectory']})[0];
    }

    if ($scope.syncPath == "") { 
      // ensure path not empty
      $scope.syncPath = path.join(home, protosync.getName());
    }
    
    console.log("using custom syncfolder " + $scope.syncPath);
    var data = {
      boxname:  $scope.host,
      protosyncthingpath: $scope.syncPath,
      username: $scope.user_username,
      email:    $scope.user_email,
      password: $scope.password
    };

    ipc.send("credentials", JSON.stringify(data));

    $scope.page = "folderdone";
  }

  $scope.openSyncFolder = function() {
    console.log("using custom syncfolder " + $scope.syncPath);
    shell.openItem($scope.syncPath);
    remote.getCurrentWindow().close();
  };

  $scope.back = function() {
    $scope.backwards = true;
    $scope.page = "boxname";
    $timeout(function() {
      $scope.backwards = false;
    }, 600);
  };

  $scope.openPasswordForgotten = function() {
    var url = "https://" + $scope.host + "/users/password/new";
    shell.openExternal(url);
  };

});
